package com.gct.system.aspect;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.gct.common.security.util.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

/**
 * 自定义公共字段填充处理器--------mybatis plus 的功能
 * @author: NewMeanning
 * @create: 2020-12-28 15:59
 */
@Slf4j
@Component
public class MyMetaObjectHandler implements MetaObjectHandler {

    /**
     * 插入操作 自动填充
     */
    @Override
    public void insertFill(MetaObject metaObject) {
        log.info("start insert fill ....");
        //获取到需要被填充的字段的值
        Object createBy = getFieldValByName("createBy", metaObject);
        Object createTime = getFieldValByName("createTime", metaObject);
        if(createBy == null){
            String username = SecurityUtils.getUsername();
            log.info("insert username:{}",username);
            this.strictInsertFill(metaObject, "createBy", String.class, username);
        }
        if(createTime == null) {
            this.strictInsertFill(metaObject, "createTime", LocalDateTime.class, LocalDateTime.now());
        }

    }
    /**
     * 修改操作 自动填充
     */
    @Override
    public void updateFill(MetaObject metaObject) {
        Object updateBy = getFieldValByName("updateBy", metaObject);
        Object updateTime = getFieldValByName("updateTime", metaObject);
        if(updateBy == null){
            this.strictInsertFill(metaObject, "updateBy", LocalDateTime.class, LocalDateTime.now());
        }
        if(updateTime == null) {
            this.strictInsertFill(metaObject, "updateTime", LocalDateTime.class, LocalDateTime.now());
        }
    }
}

