package com.gct.system.controller;


import com.gct.common.core.entity.PageRequest;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import com.gct.system.entity.SysDept;
import com.gct.system.service.SysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author NewMeanning
 * @since 2020-12-23
 */
@Api(value = "SysDeptController",description = "部门管理")
@RestController
@RequestMapping("/sysDept")
@PreAuthorize("hasAuthority('sys:rights')")
public class SysDeptController {

    @Autowired
    private SysDeptService sysDeptService;

    @ApiOperation(value = "新增", notes = "新增部门接口")
    @PostMapping("/add")
    public Result addDept(@RequestBody SysDept sysDept){
        return sysDeptService.addDept(sysDept);
    }

    @ApiOperation(value = "修改", notes = "修改部门接口")
    @PostMapping("/modify")
    public Result modifyDept(@RequestBody SysDept sysDept){
        return sysDeptService.modifyDept(sysDept);
    }

    @ApiOperation(value = "删除（批量）", notes = "批量删除部门接口")
    @PostMapping("/delete")
    public Result deleteDept(@RequestBody List<String> ids){
        boolean removeByIds = sysDeptService.removeByIds(ids);
        return  removeByIds?Result.create(CodeMsgEnum.DELETE_SUCCESS):Result.create(CodeMsgEnum.DELETE_FAIL);
    }

    @ApiOperation(value = "详情", notes = "根据id获取部门信息接口")
    @GetMapping("/{id}")
    @PreAuthorize("hasAuthority('sys:dept:select')")
    public Result findDeptById(@PathVariable String id){
        return Result.create(CodeMsgEnum.SUCCESS,sysDeptService.getById(id));
    }

    @ApiOperation(value = "分页查询", notes = "分页查询部门信息列表")
    @PostMapping("/pages")
    public Result findDeptByPages(@RequestBody PageRequest pageRequest){
        return sysDeptService.findListByPages(pageRequest);
    }

    @ApiOperation(value = "树形结构", notes = "树形结构查询")
    @GetMapping("/tree")
    public Result findTree(){
        return sysDeptService.findTree();
    }

    @ApiOperation(value = "列表", notes = "所有部门列表")
    @GetMapping("/list")
    public Result findList(){
        return sysDeptService.findList();
    }
}

