package com.gct.system.controller;


import com.alibaba.csp.sentinel.annotation.SentinelResource;
import com.gct.common.core.result.CodeMsg;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import com.gct.common.security.core.CheckTokenCaching;
import com.gct.common.security.core.JwtAuthenticatioToken;
import com.gct.common.security.util.JwtTokenUtils;
import com.gct.common.security.util.PasswordUtils;
import com.gct.common.security.util.SecurityUtils;
import com.gct.system.config.SentinelBockExceptionHandler;
import com.gct.system.config.SentinelFallbackHandler;
import com.gct.system.constant.SysConstants;
import com.gct.system.entity.SysUser;
import com.gct.system.service.SysUserService;
import com.gct.system.util.VerificationCodeImgUtil;
import com.gct.system.vo.LoginBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.compress.utils.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.imageio.ImageIO;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.IOException;

/**
 * 登录控制器
 * @author: NewMeanning
 * @create: 2020-12-25 11:36
 */
@Api(value = "SysLoginController",description = "登录")
@RestController
public class SysLoginController {

	@Autowired
	private CheckTokenCaching checkTokenCaching;
	@Autowired
	private SysUserService sysUserService;

	@Autowired
	private AuthenticationManager authenticationManager;

	@GetMapping("checkCode.jpg")
	public void captcha(HttpServletResponse response, HttpServletRequest request) throws ServletException, IOException {
		response.setHeader("Cache-Control", "no-store, no-cache");
		response.setContentType("image/jpeg");

		VerificationCodeImgUtil verificationCodeImgUtil = new VerificationCodeImgUtil();
		// 生成文字验证码
		String text = verificationCodeImgUtil.getCode();
		// 生成图片验证码
		BufferedImage image = verificationCodeImgUtil.getBuffImg();
		// 保存到验证码到 session
		request.getSession().setAttribute(SysConstants.SESSION_KEY, text);

		ServletOutputStream out = response.getOutputStream();
		ImageIO.write(image, "jpg", out);	
		IOUtils.closeQuietly(out);
	}

	/**
	 * 登录接口
	 */
	@ApiOperation(value = "登陆", notes = "登录接口")
	@PostMapping(value = "/login")
	@SentinelResource(value = "Login-Entry",blockHandlerClass = SentinelBockExceptionHandler.class,blockHandler = "handle",fallbackClass = SentinelFallbackHandler.class,fallback = "handle")
	public Result login(@RequestBody LoginBean loginBean, HttpServletRequest request) throws IOException {
		String username = loginBean.getAccount();
		String password = loginBean.getPassword();
		String checkCode = loginBean.getCheckCode();
		// 从session中获取之前保存的验证码跟前台传来的验证码进行匹配
		Object sessionCode = request.getSession().getAttribute(SysConstants.SESSION_KEY);
//		if(sessionCode == null){
//			return Result.create(CodeMsg.createCodeMsg(-1,"验证码已失效"));
//		}
//		if(!checkCode.equals(sessionCode)){
//			return Result.create(CodeMsg.createCodeMsg(-1,"验证码不正确"));
//		}
		// 用户信息
		SysUser user = sysUserService.findUserByName(username);
		// 账号不存在、密码错误
		if (user == null) {
			return Result.create(CodeMsg.createCodeMsg(-1,"账号不存在"));
		}
		if (!PasswordUtils.matches(user.getSalt(), password, user.getPassword())) {
			return Result.create(CodeMsg.createCodeMsg(-1,"密码不正确"));
		}
		// 账号锁定
		if (user.getStatus().intValue() == 0) {
			return Result.create(CodeMsg.createCodeMsg(-1,"账号已被锁定,请联系管理员"));
		}
		// 系统登录认证
		JwtAuthenticatioToken token = SecurityUtils.login(request, username, password, authenticationManager,checkTokenCaching);
		// 记录登录日志
		//sysLoginLogService.writeLoginLog(username, IPUtils.getIpAddr(request));
		return Result.create(CodeMsgEnum.SUCCESS,token);
	}

	@ApiOperation(value = "close session and token",notes = "close session and token")
	@GetMapping(value = "/loginOut")
	public Result loginOut(HttpServletRequest request){
		String token = JwtTokenUtils.getToken(request);
		checkTokenCaching.deleteTokenFromCache(token);
		return Result.ok("已退出登录!");
	}
}
