package com.gct.system.controller;


import com.gct.system.entity.SysMenu;
import com.gct.system.entity.SysOrgRole;
import com.gct.system.service.SysMenuService;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import com.gct.common.core.entity.PageRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author NewMeanning
 * @since 2020-12-23
 */
@Api(value = "SysMenuController", description = "菜单管理")
@RestController
@RequestMapping("/sysMenu")
public class SysMenuController {

    @Autowired
    private SysMenuService sysMenuService;

    @ApiOperation(value = "新增", notes = "新增菜单接口")
    @PostMapping("/add")
    public Result addMenu(@RequestBody SysMenu sysMenu) {
        return sysMenuService.createMenu(sysMenu);
    }

    @ApiOperation(value = "修改/假删除 delFlag= -1", notes = "修改菜单信息接口")
    @PostMapping("/modify")
    public Result modifyMenu(@RequestBody SysMenu sysMenu) {
        return sysMenuService.updateMenu(sysMenu);
    }

    @ApiOperation(value = "删除（批量）", notes = "删除菜单信息接口")
    @PostMapping("/delete")
    public Result deleteMenu(@RequestBody List<String> ids) {
        boolean removeByIds = sysMenuService.removeByIds(ids);
        return removeByIds ? Result.create(CodeMsgEnum.DELETE_SUCCESS) : Result.create(CodeMsgEnum.DELETE_FAIL);
    }

    @ApiOperation(value = "详情", notes = "根据id获取菜单信息接口")
    @GetMapping("/{id}")
    public Result findMenuById(@PathVariable String id) {
        return Result.create(CodeMsgEnum.SUCCESS, sysMenuService.getById(id));
    }

    @ApiOperation(value = "树形结构", notes = "树形结构查询")
    @GetMapping("/tree")
    public Result findTree(@ApiParam(value = "限制查询的最大level") @RequestParam("maxLevel") int maxLevel) {
        List<SysMenu> tree = sysMenuService.findTree(maxLevel);
        return tree==null?Result.error("权限不足"):Result.ok(tree);
    }

    @ApiOperation(value = "列表", notes = "所有菜单列表")
    @GetMapping("/list")
    public Result findList() {
        return sysMenuService.findList();
    }

    @GetMapping("/selectMenuListByUserName/{user}")
    public Result selectMenuListByUserName(@PathVariable("user") String username) {
        List<SysMenu> l = sysMenuService.selectMenuListByUserName(username,0);
        Result result = l == null || l.isEmpty() ? Result.error("权限不足") : Result.ok(l);
        return result;
    }

    @GetMapping("/selectMenuListByRole/{role}")
    public Result selectMenuListByRoleID(@PathVariable("role") String role) {
        List<SysMenu> l = sysMenuService.selectMenuListByRoleID(role,0);
        Result result = l == null || l.isEmpty() ? Result.error("权限不足") : Result.ok(l);
        return result;
    }

    @GetMapping("/selectMenuTreeByUserName/{user}/{levelLimit}")
    public Result selectMenuTreeByUserName(@PathVariable("user") String username,@PathVariable("levelLimit") int maxLevel) {
        List<SysMenu> l = sysMenuService.selectMenuTreeByUserName(username,maxLevel);
        Result result = l == null || l.isEmpty() ? Result.error("权限不足") : Result.ok(l);
        return result;
    }

    @GetMapping("/selectMenuTreeByRoleID/{role}/{levelLimit}")
    public Result selectMenuTreeByRoleID(@PathVariable("role") String role,@PathVariable("levelLimit") int maxLevel) {
        List<SysMenu> l = sysMenuService.selectMenuTreeByRoleID(role,maxLevel);
        Result result = l == null || l.isEmpty() ? Result.error("权限不足") : Result.ok(l);
        return result;
    }

    @GetMapping("/selectMenuTreeByOrgRoleID/{role}/{levelLimit}")
    public Result selectMenuTreeByOrgRoleID(@PathVariable("role") String role,@PathVariable("levelLimit") int maxLevel) {
        List<SysOrgRole> l = sysMenuService.selectMenuTreeByOrgRoleID(role,maxLevel);
        Result result = l == null || l.isEmpty() ? Result.error("权限不足") : Result.ok(l);
        return result;
    }
}

