package com.gct.system.controller;


import com.alibaba.fastjson.JSONObject;
import com.gct.system.entity.SysDept;
import com.gct.system.entity.SysRole;
import com.gct.system.service.SysDeptService;
import com.gct.system.service.SysRoleService;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import com.gct.common.core.entity.PageRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author NewMeanning
 * @since 2020-12-23
 */
@Api(value = "SysRoleController",description = "角色管理")
@RestController
@RequestMapping("/sysRole")
@PreAuthorize("hasAuthority('sys:rights')")
public class SysRoleController {

    @Autowired
    private SysRoleService sysRoleService;

    @ApiOperation(value = "新增", notes = "新增角色接口")
    @PostMapping("/add")
    public Result addRole(@RequestBody SysRole sysRole){
        /**
         * Jackson parse error
         */
//        System.out.println(sysRole);
//        return sysRoleService.addRole(sysRole);
//        String json="{\"roleName\":\"asd\",\"deptId\":\"8e4b0d9d735fe0dd7d60f14a79c7cf36\",\"roleDescription\":\"asd\",\"menuArray\":[{\"id\":\"1\",\"menuName\":\"首页\",\"menuDescription\":\"首页(╯‵□′)╯︵┻━┻\",\"icon\":\"el-icon-title\",\"menuType\":0,\"menuStatus\":1,\"sort\":1,\"createBy\":\"admin\",\"createTime\":\"2020-12-26 01:54:09\",\"updateBy\":\"admin\",\"updateTime\":\"2020-12-26 02:01:46\",\"level\":0,\"orderField\":\"1\"},{\"id\":\"2\",\"menuName\":\"系统管理\",\"menuDescription\":\"(╯‵□′)╯︵┻━┻\",\"icon\":\"el-icon-tools\",\"menuType\":0,\"menuStatus\":1,\"sort\":2,\"createBy\":\"admin\",\"createTime\":\"2020-12-26 01:55:09\",\"updateBy\":\"admin\",\"updateTime\":\"2020-12-26 02:01:48\",\"level\":0,\"orderField\":\"2\"},{\"id\":\"3\",\"menuName\":\"角色管理\",\"menuDescription\":\"(╯‵□′)╯︵┻━┻\",\"icon\":\"el-icon-user-solid\",\"url\":\"/roleManage\",\"menuType\":0,\"menuStatus\":1,\"sort\":3,\"createBy\":\"admin\",\"createTime\":\"2020-12-26 02:01:16\",\"updateBy\":\"admin\",\"updateTime\":\"2020-12-26 02:01:51\",\"parentId\":\"2\",\"level\":1,\"orderField\":\"2/3\"},{\"id\":\"4\",\"menuName\":\"部门管理\",\"menuDescription\":\"(╯‵□′)╯︵┻━┻\",\"icon\":\"el-icon-s-custom\",\"url\":\"/classManage\",\"menuType\":0,\"menuStatus\":1,\"sort\":4,\"createBy\":\"admin\",\"createTime\":\"2020-12-26 02:01:19\",\"updateBy\":\"admin\",\"updateTime\":\"2020-12-26 02:01:54\",\"parentId\":\"2\",\"level\":1,\"orderField\":\"2/4\"}]}\n";
//        SysRole sysRole1 = JSONObject.parseObject(sysRole.toJSONString(), SysRole.class);
//        System.out.println(sysRole1);
        return sysRoleService.addRole(sysRole);
//        return  Result.create(CodeMsgEnum.ADD_SUCCESS);
    }

    @ApiOperation(value = "修改角色信息", notes = "修改角色接口")
    @PostMapping("/modify")
    @PreAuthorize("!#sysRole.id.equals('1')")
    public Result modifyRole(@RequestBody SysRole sysRole){
//        SysRole sysRole1 = JSONObject.parseObject(sysRole.toJSONString(), SysRole.class);
        return sysRoleService.modifyRole(sysRole);
    }

    @ApiOperation(value = "删除（批量）", notes = "批量删除角色接口")
    @PostMapping("/delete")
    @PreAuthorize("!#ids.contains('1')")
    public Result deleteRole(@RequestBody List<String> ids){
        boolean removeByIds = sysRoleService.delRoleByIDs(ids);
        return  removeByIds?Result.create(CodeMsgEnum.DELETE_SUCCESS):Result.create(CodeMsgEnum.DELETE_FAIL);
    }

    @ApiOperation(value = "详情", notes = "根据id获取部门信息接口")
    @GetMapping("/{id}")
    public Result findRoleById(@PathVariable String id){
        return Result.create(CodeMsgEnum.SUCCESS,sysRoleService.getById(id));
    }

    @ApiOperation(value = "分页查询", notes = "分页查询用户信息列表")
    @GetMapping("/pages")
    public Result findRoleByPages(@ApiParam(name = "pageNum",value = "当前页",required = true) @RequestParam("pageNum") Integer pageNum,
                                  @ApiParam(name = "pageSize",value = "页大小",required = true) @RequestParam("pageSize") Integer pageSize,
                                  @ApiParam(name = "deptId",value = "部门ID") @RequestParam(value = "deptId",required = false) String deptId,
                                  @ApiParam(name = "roleName",value = "角色名称") @RequestParam(value = "roleName",required = false) String roleName){
        return sysRoleService.findRoleByPages(pageNum,pageSize,deptId,roleName);
    }

    @ApiOperation(value = "列表", notes = "所有角色列表")
    @GetMapping("/list")
    public Result findList(){
        return sysRoleService.findList();
    }
}

