package com.gct.system.controller;

import com.gct.system.entity.SysUser;
import com.gct.system.service.SysUserService;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import com.gct.common.core.entity.PageRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author NewMeanning
 * @since 2020-12-19
 */
@Api(value = "SysUserController",description = "用户管理")
@RestController
@RequestMapping("/sysUser")
//@PreAuthorize("hasAuthority('sys:rights')")
public class SysUserController {

    @Autowired
    private SysUserService sysUserService;

    @ApiOperation(value = "新增", notes = "新增用户接口")
    @PreAuthorize("hasAuthority('sys:user:add')")
    @PostMapping("/add")
    public Result addUser(@RequestBody SysUser sysUser){
        return sysUserService.createUser(sysUser);
    }

    @ApiOperation(value = "修改/假删除 delFlag= -1", notes = "修改用户信息接口")
    @PreAuthorize("hasAuthority('sys:user:modify')")
    @PostMapping("/modify")
    public Result modifyUser(@RequestBody SysUser sysUser){
        return sysUserService.updateUser(sysUser);
    }

    @ApiOperation(value = "删除（批量）", notes = "删除用户信息接口")
    @PreAuthorize("hasAuthority('sys:user:delete')")
    @PostMapping("/delete")
    public Result deleteUser(@RequestBody List<String> ids){
        boolean removeByIds = sysUserService.removeByIds(ids);
        return  removeByIds?Result.create(CodeMsgEnum.DELETE_SUCCESS):Result.create(CodeMsgEnum.DELETE_FAIL);
    }

    @ApiOperation(value = "详情", notes = "根据id获取用户信息接口")
    @PreAuthorize("hasAuthority('sys:user:select')")
    @GetMapping("/{id}")
    public Result findUserById(@PathVariable String id){
        return Result.create(CodeMsgEnum.SUCCESS,sysUserService.getById(id));
    }

    @ApiOperation(value = "详情", notes = "根据用户名获取用户信息接口")
    //@PreAuthorize("hasAuthority('sys:user:select')")
    @GetMapping("/u/{username}")
    public Result findUserByUsername(@PathVariable String username){
        return Result.create(CodeMsgEnum.SUCCESS,sysUserService.findUserByName(username));
    }

    @ApiOperation(value = "复杂分页查询", notes = "复杂分页查询用户信息列表")
    @PreAuthorize("hasAuthority('sys:user:select')")
    @PostMapping("/pagesPlus")
    public Result findUserByPagesPlus(@RequestBody PageRequest pageRequest){
        return sysUserService.finUserListByPagesPlus(pageRequest);
    }

    @ApiOperation(value = "分页查询", notes = "分页查询用户信息列表")
    @PreAuthorize("hasAuthority('sys:user:select')")
    @GetMapping("/pages")
    public Result findUserByPages(
            @ApiParam(name = "pageNum",value = "当前页",required = true) @RequestParam("pageNum") Integer pageNum,
            @ApiParam(name = "pageSize",value = "页大小",required = true) @RequestParam("pageSize") Integer pageSize,
            @ApiParam(name = "deptId",value = "部门ID") @RequestParam(value = "deptId",required = false) String deptId,
            @ApiParam(name = "roleId",value = "角色ID") @RequestParam(value = "roleId",required = false) String roleId,
            @ApiParam(name = "nickName",value = "昵称") @RequestParam(value = "nickName",required = false) String nickName
    ){
        return sysUserService.findUserListByPages(pageNum,pageSize,deptId,roleId,nickName);
    }






}

