package com.gct.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.core.util.QueryUtil;
import com.gct.system.entity.SysDept;
import com.gct.system.mapper.SysDeptMapper;
import com.gct.system.service.SysDeptService;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import com.gct.common.core.entity.PageRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author NewMeanning
 * @since 2020-12-23
 */
@Service
public class SysDeptServiceImpl extends ServiceImpl<SysDeptMapper, SysDept> implements SysDeptService {

    @Autowired
    private SysDeptMapper sysDeptMapper;

    @Override
    public Result addDept(SysDept sysDept) {
        String parentId = sysDept.getParentId();
        Integer sort = sysDeptMapper.getMaxSort()+1;
        sysDept.setSort(sort);
        if(parentId!=null&&!"".equals(parentId)&&!"0".equals(parentId)){
            SysDept parent = this.getById(parentId);
            sysDept.setLevel(parent.getLevel()+1);
            sysDept.setOrderField(parent.getOrderField()+"/"+sort);
        }else{
            sysDept.setLevel(0);
            sysDept.setOrderField(String.valueOf(sort));
        }
        int insert = sysDeptMapper.insert(sysDept);
        return insert==1?Result.create(CodeMsgEnum.ADD_SUCCESS):Result.create(CodeMsgEnum.ADD_FAIL);
    }

    @Override
    public Result modifyDept(SysDept sysDept) {
        int update = sysDeptMapper.updateById(sysDept);
        return update==1?Result.create(CodeMsgEnum.MODIFY_SUCCESS):Result.create(CodeMsgEnum.MODIFY_FAIL);
    }

    @Override
    public Result findListByPages(PageRequest pageRequest) {
        QueryUtil<SysDept> queryUtil = new QueryUtil<>();
        QueryWrapper<SysDept> sysDeptQueryWrapper = new QueryWrapper<>();
        try {
            sysDeptQueryWrapper = queryUtil.setQueryWrapper(pageRequest);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Page<SysDept> userPage = sysDeptMapper.selectPage(queryUtil.setPage(pageRequest),sysDeptQueryWrapper);
        return Result.ok(userPage);
    }

    @Override
    public Result findTree() {
        QueryWrapper<SysDept> wrapper = new QueryWrapper<>();
        wrapper.eq("parent_id", "")
                .or()
                .eq("parent_id", "0")
                .or()
                .isNull("parent_id");
        // 得到一级节点资源列表
        List<SysDept> resources = sysDeptMapper.selectList(wrapper);
        if (resources != null && resources.size() > 0) {
            resources.forEach(this::findAllChild);
        }
        return Result.ok(resources);
    }

    @Override
    public List<String> findAllIdById(String id) {
        List<String> list = new ArrayList<>();
        findIds(id,list);
        return list;
    }

    @Override
    public Result findList() {
        QueryWrapper<SysDept> wrapper = new QueryWrapper<>();
        wrapper.orderByAsc("order_field");
        List<SysDept> list = sysDeptMapper.selectList(wrapper);
        return Result.ok(list);
    }

    /**
     * 递归查询子部门
     * @param sysDept
     */
    private void findAllChild(SysDept sysDept) {
        QueryWrapper<SysDept> wrapper = new QueryWrapper<>();
        wrapper.eq("parent_id", sysDept.getId());
        // 首次进入这个方法时，查出的是二级节点列表
        // 递归调用时，就能依次查出三、四、五等等级别的节点列表，
        // 递归能实现不同节点级别的无限调用,这个层次不易太深，否则有性能问题
        List<SysDept> resources = sysDeptMapper.selectList(wrapper);
        sysDept.setChildren(resources);
        if (resources != null && resources.size() > 0) {
            resources.forEach(this::findAllChild);
        }
    }

    /**
     * 递归查询所有子部门Id
     * @param id
     * @param list
     */
    private void findIds(String id,List<String> list){
        list.add(id);
        QueryWrapper<SysDept> wrapper = new QueryWrapper<>();
        wrapper.eq("parent_id", id);
        List<SysDept> resources = sysDeptMapper.selectList(wrapper);
        if (resources != null && resources.size() > 0) {
            resources.stream().forEach(sysDept -> {
                this.findIds(sysDept.getId(),list);
            });
        }
    }

}
