package com.gct.system.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.system.constant.SysConstants;
import com.gct.system.entity.SysMenu;
import com.gct.system.entity.SysOrgRole;
import com.gct.system.mapper.SysMenuMapper;
import com.gct.system.mapper.SysOrgRoleMapper;
import com.gct.system.service.SysMenuService;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author NewMeanning
 * @since 2020-12-23
 */
@Service
public class SysMenuServiceImpl extends ServiceImpl<SysMenuMapper, SysMenu> implements SysMenuService {

    @Autowired
    private SysMenuMapper sysMenuMapper;
    @Autowired
    private SysOrgRoleMapper sysOrgRoleMapper;

    @Override
    public Result createMenu(SysMenu sysMenu) {
        String parentId = sysMenu.getParentId();
        Integer sort = sysMenuMapper.getMaxSort() + 1;
        sysMenu.setSort(sort);
        if (parentId != null && !"".equals(parentId) && !"0".equals(parentId)) {
            SysMenu parent = this.getById(parentId);
            sysMenu.setLevel(parent.getLevel() + 1);
            sysMenu.setOrderField(parent.getOrderField() + "/" + sort);
        } else {
            sysMenu.setLevel(0);
            sysMenu.setOrderField(String.valueOf(sort));
        }
        int insert = sysMenuMapper.insert(sysMenu);
        return insert == 1 ? Result.create(CodeMsgEnum.ADD_SUCCESS) : Result.create(CodeMsgEnum.ADD_FAIL);
    }

    @Override
    public Result updateMenu(SysMenu sysMenu) {
        int update = sysMenuMapper.updateById(sysMenu);
        return update == 1 ? Result.create(CodeMsgEnum.ADD_SUCCESS) : Result.create(CodeMsgEnum.ADD_FAIL);
    }

    @Override
    public List<SysMenu> findByUser(String username) {

        if (username == null || "".equals(username) || SysConstants.ADMIN.equalsIgnoreCase(username)) {
            return sysMenuMapper.selectList(new QueryWrapper<>());
        }
        return sysMenuMapper.findByUserName(username);
    }

    @Override
    public Result findTree() {
        QueryWrapper<SysMenu> wrapper = new QueryWrapper<>();
        wrapper.eq("parent_id", "")
                .or()
                .eq("parent_id", "0")
                .or()
                .isNull("parent_id");
        // 得到一级节点资源列表
        List<SysMenu> resources = sysMenuMapper.selectList(wrapper);
        if (resources != null && resources.size() > 0) {
            resources.forEach(this::findAllChild);
        }
        return Result.ok(resources);
    }


    @Override
    public List<SysMenu> findTree(int maxLevel){
        List<SysMenu> list = sysMenuMapper.findAllByLevelLimit(maxLevel);
        return tranlateMenuListToTree(list);
    }

    @Override
    public Result findList() {
        QueryWrapper<SysMenu> wrapper = new QueryWrapper<>();
        wrapper.orderByAsc("order_field");
        List<SysMenu> list = sysMenuMapper.selectList(wrapper);
        return Result.ok(list);
    }

    @Override
    public List<SysMenu> selectMenuListByUserName(String username, int maxLevel) {
        if (username == null || username.isEmpty()) return null;
        return sysMenuMapper.finByUserName(username, maxLevel);
    }

    @Override
    public List<SysMenu> selectMenuListByRoleID(String role, int maxLevel) {
        if (role == null || role.isEmpty()) return null;
        return sysMenuMapper.findByRoleID(role, maxLevel);
    }

    @Override
    public List<SysMenu> selectMenuTreeByUserName(String username, int maxLevel) {
        List<SysMenu> sysMenus = selectMenuListByUserName(username, maxLevel);
        return tranlateMenuListToTree(sysMenus);
    }
    @Override
    public List<SysMenu> selectMenuTreeByRoleID(String role, int maxLevel) {
        List<SysMenu> sysMenus = selectMenuListByRoleID(role, maxLevel);
        return tranlateMenuListToTree(sysMenus);
    }

    @Override
    public List<SysOrgRole> selectMenuTreeByOrgRoleID(String role, int maxLevel) {
        if (role == null || role.isEmpty()) return null;
        List<SysOrgRole> list = sysOrgRoleMapper.findByOrgRoleID(role);
        return tranlateMenuOrgRoleListToTree(list);
    }

    private List<SysMenu> tranlateMenuListToTree(List<SysMenu> sysMenus) {
        Map<String, SysMenu> sysMenuMap = new HashMap<>();
        List<SysMenu> sysMenuTree = new ArrayList<>();
        for (SysMenu sysMenu : sysMenus)
            sysMenuMap.put(sysMenu.getId(), sysMenu);
        for (SysMenu sysMenu : sysMenus) {
            if (isTopParent(sysMenu.getParentId(), sysMenu.getLevel()))
                sysMenuTree.add(sysMenu);
            else {
                SysMenu parent = sysMenuMap.get(sysMenu.getParentId());
                if (parent == null) {
                    log.error("the parent id is not exited:{}", new Throwable("the parent-id:" + sysMenu.getParentId()));
                    continue;
                }
                if (parent.getChildren() == null) parent.setChildren(new ArrayList<>());
                parent.getChildren().add(sysMenu);
            }
        }
        return sysMenuTree.size() == 0 ? null : sysMenuTree;
    }
    private List<SysOrgRole> tranlateMenuOrgRoleListToTree(List<SysOrgRole> sysOrgRoles) {
        Map<String, SysOrgRole> sysMenuMap = new HashMap<>();
        List<SysOrgRole> sysMenuTree = new ArrayList<>();
        List<String> orgRoleTree = new ArrayList<>();
        for (SysOrgRole sysOrgRole : sysOrgRoles)
            sysMenuMap.put(sysOrgRole.getOrgId(), sysOrgRole);
        sysMenuMap.values().stream().forEach(item->{
            JSONArray objects = JSONObject.parseArray(item.getOrgId());
            objects.forEach(x-> orgRoleTree.add(x.toString()));
        });
        SysOrgRole sysOrgRole = new SysOrgRole();
        sysOrgRole.setOrgId(orgRoleTree.toString());
        sysMenuTree.add(sysOrgRole);
        return sysMenuTree.size() == 0 ? null : sysMenuTree;
    }
    private boolean isTopParent(String parentID, int level) {
        if (parentID == null || parentID.isEmpty()) return true;
        if (level == 0) return true;
        return false;
    }

    /**
     * 递归查询子菜单
     *
     * @param sysDept
     */
    private void findAllChild(SysMenu sysDept) {
        QueryWrapper<SysMenu> wrapper = new QueryWrapper<>();
        wrapper.eq("parent_id", sysDept.getId());
        // 首次进入这个方法时，查出的是二级节点列表
        // 递归调用时，就能依次查出三、四、五等等级别的节点列表，
        // 递归能实现不同节点级别的无限调用,这个层次不易太深，否则有性能问题
        List<SysMenu> resources = sysMenuMapper.selectList(wrapper);
        sysDept.setChildren(resources);
        if (resources != null && resources.size() > 0) {
            resources.forEach(this::findAllChild);
        }
    }
}
