package com.gct.system.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.system.entity.SysMenu;
import com.gct.system.entity.SysOrgRole;
import com.gct.system.entity.SysRole;
import com.gct.system.mapper.SysRoleMapper;
import com.gct.system.service.SysOrgRoleService;
import com.gct.system.service.SysRoleService;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author NewMeanning
 * @since 2020-12-23
 */
@Service
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {


    @Autowired
    private SysRoleMapper sysRoleMapper;

    @Autowired
    private SysOrgRoleService sysOrgRoleService;

    @Override
    @Transactional
    public Result addRole(SysRole sysRole) {
        if (sysRole.getId() != null && !sysRole.getId().isEmpty()) sysRoleMapper.deleteById(sysRole.getId());
        if (sysRoleMapper.insert(sysRole) != 1) return Result.create(CodeMsgEnum.ADD_FAIL);
        if (!delRoleMenuRelationByRoleID(sysRole.getId())) return Result.create(CodeMsgEnum.ADD_FAIL);
        if (!modifyRoleMenuRelation(sysRole.getMenuArray(), sysRole.getId()))
            return Result.create(CodeMsgEnum.ADD_FAIL);
        if (!modifyOrgRoleRelation(sysRole.getDataTreeArray(), sysRole.getId()))
            return Result.create(CodeMsgEnum.ADD_FAIL);
        return Result.create(CodeMsgEnum.ADD_SUCCESS);
    }

    public List<Map<String, Object>> selectRoleMenuRelationByRoleID(String role) {
        if (role == null || role.isEmpty()) return null;
        return sysRoleMapper.selectRoleMenuRelationByRoleID(role);
    }

    private boolean modifyRoleMenuRelation(SysMenu[] menus, String role) {
        if (menus == null || menus.length == 0) return true;

        int insertedCout = 0;
        List<String> menuids = new ArrayList<>();
        for (SysMenu menu : menus) {
            menuids.add(menu.getId());
        }
        insertedCout = sysRoleMapper.insertRoleMenuRelationBatch(role, menuids);
        if (insertedCout != menus.length)
            log.error("insert failed for menus", new Throwable("sus:" + insertedCout + "failed:" + (menus.length - insertedCout)));
        return true;
    }

    private List<SysOrgRole> child (SysOrgRole[] orgRoles){
        Map<String, SysOrgRole> map = new HashMap<>();
        Arrays.stream(orgRoles).forEach(orgRole->map.put(orgRole.getOrgId(),orgRole));
        for (SysOrgRole orgRole : orgRoles) {
            List<SysOrgRole> children = orgRole.getChildren();
            if (children != null && children.size()>0){
               children.forEach(x->eraseChild(map,x));
            }
        }
        List<SysOrgRole> list = new ArrayList<>();
        list.addAll(map.values());
        return list;
    }

    private void eraseChild(Map<String, SysOrgRole>map,SysOrgRole orgRole){
        if (!(orgRole.getChildren()==null || orgRole.getChildren().size()==0)){
            orgRole.getChildren().forEach(x->eraseChild(map,x));
        }
        map.remove(orgRole.getOrgId());
    }

    private boolean modifyOrgRoleRelation(SysOrgRole[] orgRoles, String role) {

        if (orgRoles == null || orgRoles.length == 0) return true;

        List<SysOrgRole> child = child(orgRoles);
        SysOrgRole sysOrgRole = new SysOrgRole();
        sysOrgRole.setRoleId(role);
        JSONArray ja = new JSONArray();
        for (SysOrgRole orgRole : child) {
            ja.add(orgRole.getOrgId());
        }
        sysOrgRole.setOrgId(ja.toJSONString());
        sysOrgRoleService.save(sysOrgRole);
        return true;
    }

    @Override
    @Transactional
    public boolean delRoleByIDs(List<String> ids) {
        if (sysRoleMapper.deleteBatchIds(ids) <= 0) return false;
        if (!delOrgRole(ids)) return false;
        if (!deleteRoleMenuRelationByRoleID(ids)) return false;

        return true;

    }

    private boolean delOrgRole(List<String> ids){
        if (ids == null || ids.size() <=0 || ids.isEmpty()) return false;
        QueryWrapper<SysOrgRole> wrapper = new QueryWrapper<SysOrgRole>().in("role_id",ids);
        return sysOrgRoleService.remove(wrapper);
    }

    @Override
    public boolean delRoleMenuRelationByRoleID(String role) {
        if (role == null || role.isEmpty()) return true;
        List<Map<String, Object>> relations = selectRoleMenuRelationByRoleID(role);
        if (relations != null && relations.size() > 0)
            if (sysRoleMapper.delRoleMenuRelationByRoleID(role) <= 0)
                return false;

        return true;
    }

    @Override
    @Transactional
    public Result modifyRole(SysRole sysRole) {
        if(sysRole!=null) sysRole.setUpdateTime(LocalDateTime.now());//fill update time
        if (sysRoleMapper.updateById(sysRole) != 1) return Result.create(CodeMsgEnum.MODIFY_FAIL);
        if (updateOrgRole(sysRole.getDataTreeArray(),sysRole.getId())) return Result.create(CodeMsgEnum.MODIFY_FAIL);
        if (!delRoleMenuRelationByRoleID(sysRole.getId())) return Result.create(CodeMsgEnum.MODIFY_FAIL);
        if (!modifyRoleMenuRelation(sysRole.getMenuArray(), sysRole.getId()))
            return Result.create(CodeMsgEnum.MODIFY_FAIL);
        return Result.create(CodeMsgEnum.MODIFY_SUCCESS);
    }

    private boolean updateOrgRole(SysOrgRole[] dataTreeArray , String roleId){
        if (dataTreeArray==null || dataTreeArray.length==0) return false;
        QueryWrapper<SysOrgRole> wrapper = new QueryWrapper<SysOrgRole>().eq("role_id",roleId);
        List<SysOrgRole> child = child(dataTreeArray);
        SysOrgRole sysOrgRole = new SysOrgRole();
        sysOrgRole.setRoleId(roleId);
        JSONArray ja = new JSONArray();
        for (SysOrgRole orgRole : child) {
            ja.add(orgRole.getOrgId());
        }
        sysOrgRole.setOrgId(ja.toJSONString());

        sysOrgRoleService.update(sysOrgRole,wrapper);

        return false;
    }

    @Override
    public Result findRoleByPages(Integer pageNum, Integer pageSize, String deptId, String roleName) {
        Page<SysRole> page = new Page<>(pageNum, pageSize);
        QueryWrapper<SysRole> wrapper = new QueryWrapper<>();
        if (deptId != null && !"".equals(deptId)) {
            wrapper.eq("dept_id", deptId);
        }
        if (roleName != null && !"".equals(roleName)) {
            wrapper.like("role_name", roleName);
        }
        wrapper.orderByAsc("create_time");
        Page<SysRole> selectPage = sysRoleMapper.selectPage(page, wrapper);
        return Result.ok(selectPage);
    }

    @Override
    public Result findList() {
        List<SysRole> list = sysRoleMapper.selectList(new QueryWrapper<>());
        return Result.ok(list);
    }

    @Override
    public boolean deleteRoleMenuRelationByRoleID(List<String> roles) {
        if (roles == null || roles.isEmpty() || roles.size() == 0) return true;

        int i = sysRoleMapper.delRoleMenuRelationByRoleIDBatch(roles);
        return true;
    }
}
