package com.gct.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.util.QueryUtil;
import com.gct.common.security.util.PasswordUtils;
import com.gct.system.constant.SysConstants;
import com.gct.system.entity.SysMenu;
import com.gct.system.entity.SysUser;
import com.gct.system.mapper.SysUserMapper;
import com.gct.system.service.SysDeptService;
import com.gct.system.service.SysMenuService;
import com.gct.system.service.SysUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.core.result.CodeMsg;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import com.gct.common.core.entity.PageRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author NewMeanning
 * @since 2020-12-19
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Autowired
    private SysUserMapper sysUserMapper;


    @Autowired
    private SysMenuService sysMenuService;


    @Autowired
    private SysDeptService sysDeptService;

    @Override
    @Transactional
    public Result createUser(SysUser sysUser) {
        if(sysUser.getPassword() != null) {
            String salt = PasswordUtils.getSalt();//生成加密盐
            if(this.findUserByName(sysUser.getUsername()) != null) { //用户名验证
                return Result.create(CodeMsg.createCodeMsg(-1,"用户名已存在"));
            }
            String password = PasswordUtils.encode(sysUser.getPassword(), salt);
            sysUser.setSalt(salt);
            sysUser.setPassword(password);
        }
        //添加用户
        int insert = sysUserMapper.insert(sysUser);
        //添加用户-角色 关联表
//        SysUser userByName = this.findUserByName(sysUser.getUsername());
//        sysUserMapper.insertUserRole(userByName.getId(),sysUser.getRoleId());
        return insert==1?Result.create(CodeMsgEnum.ADD_SUCCESS):Result.create(CodeMsgEnum.ADD_FAIL);
    }

    @Override
    public Result updateUser(SysUser sysUser) {
        SysUser user = this.getById(sysUser.getId());
        if(SysConstants.ADMIN.equalsIgnoreCase(user.getUsername())) { //判断是否修改的管理员
            return Result.create(CodeMsg.createCodeMsg(-1,"超级管理员不允许修改"));
        }
        if(!sysUser.getPassword().equals(user.getPassword())) { //判断是否修改密码
            String salt = PasswordUtils.getSalt();
            String password = PasswordUtils.encode(sysUser.getPassword(), salt);
            sysUser.setSalt(salt);
            sysUser.setPassword(password);
        }
        int update = sysUserMapper.updateById(sysUser);
        return update==1?Result.create(CodeMsgEnum.MODIFY_SUCCESS):Result.create(CodeMsgEnum.MODIFY_FAIL);
    }

    @Override
    public Result finUserListByPagesPlus(PageRequest pageRequest) {
        QueryUtil<SysUser> queryUtil = new QueryUtil<>();
        QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
        try {
            sysUserQueryWrapper = queryUtil.setQueryWrapper(pageRequest);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Page<SysUser> userPage = sysUserMapper.selectPage(queryUtil.setPage(pageRequest), sysUserQueryWrapper);
        return Result.ok(userPage);

    }

    @Override
    public SysUser findUserByName(String username) {
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();   //创建查询queryWrapper
        queryWrapper.eq("username",username);
        return sysUserMapper.selectOne(queryWrapper);
    }

    @Override
    public Result findUserListByPages(Integer pageNum, Integer pageSize, String deptId, String roleId, String nickName) {
        List<String> deptIds = null;
        if(deptId!=null&& !"".equals(deptId)){
            deptIds = sysDeptService.findAllIdById(deptId);
        }
        Page<SysUser> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        paramPage.addOrder(OrderItem.asc("create_time"));
        IPage<SysUser> page = sysUserMapper.selectUserList(paramPage, deptIds, roleId, nickName);
        return Result.ok(page);
    }

    @Override
    public Set<String> findPermissions(String username) {
        Set<String> perms = new HashSet<>();
        List<SysMenu> sysMenus = sysMenuService.findByUser(username);
        for(SysMenu sysMenu:sysMenus) {
            if(sysMenu.getPerms() != null && !"".equals(sysMenu.getPerms())) {
                perms.add(sysMenu.getPerms());
            }
        }
        return perms;
    }
}
